      subroutine radiative_forcing(Nk1,Nk2,f1,f2,rf)
      implicit none
      include 'max.inc'
c     
c     Purpose: to compute a radiative forcing from input fluxes
c     
c     Input:
c       + Nk1: number of wavenumber points for array "f1"
c       + Nk2: number of wavenumber points for array "f2"
c       + f1: flux 1 [W/m^2/inv. cm]
c       + f2: flux 2 [W/m^2/inv. cm]
c       
c     Output:
c       + rf: flux 2 - flux 1 [W/m^2/inv. cm]
c     
c     I/O
      integer Nk1,Nk2
      double precision f1(1:Nk_mx,1:2)
      double precision f2(1:Nk_mx,1:2)
      double precision rf(1:Nk_mx,1:2)
c     temp
      integer ik,Nk
      double precision epsilon_nu
c     parameters
      parameter(epsilon_nu=1.0D-12)
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine radiative_forcing'

      if (Nk1.ne.Nk2) then
         call error(label)
         write(*,*) 'Nk1=',Nk1
         write(*,*) 'Nk2=',Nk2
         write(*,*) 'should not be different'
         stop
      else
         Nk=Nk1
      endif
      
      do ik=1,Nk
         if (dabs(f1(ik,1)-f2(ik,1)).ge.epsilon_nu) then
            call error(label)
            write(*,*) 'nu1(',ik,')=',f1(ik,1)
            write(*,*) 'nu2(',ik,')=',f2(ik,1)
            write(*,*) 'diff=',dabs(f1(ik,1)-f2(ik,1))
            write(*,*) 'should not be different'
            stop
         else
            rf(ik,1)=f1(ik,1)
            rf(ik,2)=f2(ik,2)-f1(ik,2)
         endif
      enddo ! ik

      return
      end


      
      subroutine radiative_forcing_profile(Nlev1,Nlev2,f1,f2,rf)
      implicit none
      include 'max.inc'
c     
c     Purpose: to compute a radiative forcing from input flux profiles
c     
c     Input:
c       + Nlev1: number of atmospheric levels for flux "f1"
c       + Nlev2: number of atmospheric levels for flux "f2"
c       + f1: flux 1 profile [W/m^2]
c       + f2: flux 2 profile [W/m^2]
c       
c     Output:
c       + rf: flux 2 - flux 1 [W/m^2]
c     
c     I/O
      integer Nlev1,Nlev2
      double precision f1(1:Nlev_mx,1:2)
      double precision f2(1:Nlev_mx,1:2)
      double precision rf(1:Nlev_mx,1:2)
c     temp
      integer ilev,Nlev
      double precision epsilon_p
c     parameters
      parameter(epsilon_p=1.0D-12)
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine radiative_forcing_profile'

      if (Nlev1.ne.Nlev2) then
         call error(label)
         write(*,*) 'Nlev1=',Nlev1
         write(*,*) 'Nlev2=',Nlev2
         write(*,*) 'should not be different'
         stop
      else
         Nlev=Nlev1
      endif
      
      do ilev=1,Nlev
         if (dabs(f1(ilev,1)-f2(ilev,1)).ge.epsilon_p) then
            call error(label)
            write(*,*) 'nu1(',ilev,')=',f1(ilev,1)
            write(*,*) 'nu2(',ilev,')=',f2(ilev,1)
            write(*,*) 'diff=',dabs(f1(ilev,1)-f2(ilev,1))
            write(*,*) 'should not be different'
            stop
         else
            rf(ilev,1)=f1(ilev,1)
            rf(ilev,2)=f2(ilev,2)-f1(ilev,2)
         endif
      enddo ! ilev

      return
      end
      


      subroutine convert(Nk,flux)
      implicit none
      include 'max.inc'
c     
c     Purpose: to convert from [W/m^2/inv. cm] to [W/m^2/micrometer]
c     
c     Input:
c       + Nk: number of wavenumber points for array "flux"
c       + flux: input flux [inv.cm | W/m^2/inv. cm]
c     
c     Output:
c       + flux: output flux [micrometer | W/m^2/micrometer]
c     
c     I/O
      integer Nk
      double precision flux(1:Nk_mx,1:2)
c     temp
      integer ik,j
      double precision dnu_dlambda
      double precision tmp(1:Nk_mx,1:2)
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine convert'

      do ik=1,Nk
         tmp(ik,2)=flux(ik,2)*dabs(dnu_dlambda(flux(ik,1))) ! W/m^2/inv.cm -> W/m^2/micrometer
c     Debug
c         if (tmp(ik,2).lt.0.0D+0) then
c            call error(label)
c            write(*,*) 'tmp(',ik,',2)=',tmp(ik,2)
c            write(*,*) 'flux(',ik,',2)=',flux(ik,2)
c            write(*,*) 'flux(',ik,',1)=',flux(ik,1)
c            write(*,*) 'dnu_dlambda=',dnu_dlambda(flux(ik,1))
c            stop
c         endif
c     Debug
         call nu2lambda(flux(ik,1),tmp(ik,1)) ! inv.cm -> micrometer
      enddo                     ! ik
      do ik=1,Nk
         do j=1,2
            flux(ik,j)=tmp(Nk-ik+1,j)
         enddo                  ! j
      enddo                     ! ik

      return
      end
      


      subroutine spectral_integration(Nk,flux,integral)
      implicit none
      include 'max.inc'
c     
c     Purpose: to integrate a spectral signal
c     
c     Input:
c       + Nk: number of wavenumber points for array "flux"
c       + flux: input spectral signal
c     
c     Output:
c       + integral: integral of the spectral signal, over the available spectral range
c     
c     I/O
      integer Nk
      double precision flux(1:Nk_mx,1:2)
      double precision integral
c     temp
      integer ik
      double precision sum
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine spectral_integration'

      sum=0.0D+0
      do ik=2,Nk
         sum=sum
     &        +(flux(ik,2)+flux(ik-1,2))
     &        *(flux(ik,1)-flux(ik-1,1))
     &        /2.0D+0
      enddo                     ! ik
      integral=sum

      return
      end
      
