c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine dimer_CIA_optical_thickness(
     &     ndt,nu_min,nu_max,np,
     &     T,cia,
     &     z1,z2,p1,p2,T1,T2,nu,
     &     tau)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to compute the optical thickness due to the CIA continuum
c
c     Input:
c       + ndt: number of spectral intervals / temperature points the data is provided for
c       + nu_min: lower wavenumber (inv. cm) for each data interval
c       + nu_max: higher wavenumber (inv. cm) for each data interval
c       + np: number of wavenumber points for each data interval
c       + T: temperature (K) for each data interval
c       + cia: CIA data for each data interval, for each wavenumber point
c         - column 1: wavenumber (inv. cm)
c         - column 2: CIA cross-section (cm^5.molecule^-2)
c       + z1: lower altitude (m)
c       + z2: higher altitude (m)
c       + p1: lower pressure (atm) @ z1
c       + p2: higher pressure (atm) @ z2
c       + T1: lower temperature (K) @ z1
c       + T2: higher temperature (K) @ z2
c       + nu: wavenumber (inv. cm)
c
c     Output:
c       + tau: CIA absorption optical thickness
c
c     I/O
      double precision nu_min(1:Ndt_int_mx)
      double precision nu_max(1:Ndt_int_mx)
      integer ndt,np(1:Ndt_int_mx)
      double precision T(1:Ndt_int_mx)
      double precision cia(1:Ndt_int_mx,1:Np_mx,1:2)
      double precision z1,z2
      double precision p1,p2
      double precision T1,T2
      double precision nu ! inv. cm
      double precision tau
c     temp
      double precision sigma1,sigma2
      double precision k1,k2
      logical is_nan
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine dimer_CIA_optical_thickness'
c     
      call cia_interpolation(ndt,nu_min,nu_max,
     &     np,T,cia,
     &     nu,T1,sigma1)
      if (sigma1.lt.0.0D+0) then
         sigma1=0.0D+0
      endif
      call cia_conversion(sigma1,p1,T1,k1) ! [k1]=m^-1
c     
      call cia_interpolation(ndt,nu_min,nu_max,
     &     np,T,cia,
     &     nu,T2,sigma2)
      if (sigma2.lt.0.0D+0) then
         sigma2=0.0D+0
      endif
      call cia_conversion(sigma2,p2,T2,k2) ! [k2]=m^-1
c
      call tau_from_zk(z1,z2,k1,k2,tau)
      if (tau.lt.0.0D+0) then
         tau=0.0D+0
      endif
c
      call test_nan_dble(tau,is_nan)
      if (is_nan) then
         call error(label)
         write(*,*) 'tau=',
     &        tau
         write(*,*) 'nu=',nu
         write(*,*) 'z1=',z1
         write(*,*) 'z2=',z2
         write(*,*) 'p1=',p1,' atm'
         write(*,*) 'T1=',T1,' K'
         write(*,*) 'sigma1=',sigma1,' cm^5.molecule^-2'
         write(*,*) 'k1=',k1,' m^-1'
         write(*,*) 'p2=',p2,' atm'
         write(*,*) 'T2',T2,' K'
         write(*,*) 'sigma2=',sigma2,' cm^5.molecule^-2'
         write(*,*) 'k2=',k2,' m^-1'
         stop
      endif

      return
      end



      subroutine CO2_continuum_optical_thickness(
     &     z1,z2,p1,p2,T1,T2,nu,
     &     tau_co2_cia)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to compute the optical thickness due to the CO2 dimer CIA continuum
c
c     Input:
c       + z1: lower altitude (m)
c       + z2: higher altitude (m)
c       + p1: lower CO2 partial pressure (atm) @ z1
c       + p2: higher CO2 partial pressure (atm) @ z2
c       + T1: lower temperature (K) @ z1
c       + T2: higher temperature (K) @ z2
c       + nu: wavenumber (inv. cm)
c
c     Output:
c       + tau_co2_cia: CO2 continuum absorption optical thickness
c
c     I/O
      double precision z1,z2
      double precision p1,p2
      double precision T1,T2
      double precision nu ! inv. cm
      double precision tau_co2_cia
c     temp
      integer ilev,ilay,ilev1,ilev2,band
      double precision amg1,amg2
      double precision cont1,cont2
      double precision ka_cont1,ka_cont2
      double precision nuc
      logical is_nan
      double precision kbar(1:Nlay_mx,1:Nb_mx)
      character*(Nchar_mx) kbar_file
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine CO2_continuum_optical_thickness'

      call amagat(p1,T1,amg1) ! [amg1]=amg
      call amagat(p2,T2,amg2) ! [amg2]=amg
c     CO2 dimer (Borysow routine)
      call getspc(T1,nu,cont1) ! [cont1]=cm^-1.amg^-2
      ka_cont1=cont1*1.0D+2*amg1**2.0D+0 ! m^-1
      call getspc(T2,nu,cont2) ! [cont2]=cm^-1.amg^-2
      ka_cont2=cont2*1.0D+2*amg2**2.0D+0 ! m^-1
      call tau_from_zk(z1
     &     ,z2
     &     ,ka_cont1
     &     ,ka_cont2
     &     ,tau_co2_cia)
      call test_nan_dble(tau_co2_cia,is_nan)
      if (is_nan) then
         call error(label)
         write(*,*) 'tau_co2_cia=',
     &        tau_co2_cia
         write(*,*) 'z1=',z1
         write(*,*) 'z2=',z2
         write(*,*) 'p1=',p1,' Pa'
         write(*,*) 'T1=',T1,' K'
         write(*,*) 'amg1=',amg1,' amagats'
         write(*,*) 'cont1=',cont1,' cm^-1.amg^-2'
         write(*,*) 'ka_cont1=',ka_cont1,' m^-1'
         write(*,*) 'p2=',p2,' Pa'
         write(*,*) 'T2',T2,' K'
         write(*,*) 'amg2=',amg2,' amagats'
         write(*,*) 'cont2=',cont2,' cm^-1.amg^-2'
         write(*,*) 'ka_cont2=',ka_cont2,' m^-1'
         stop
      endif

      return
      end



      subroutine H2O_continuum_optical_thickness(
     &     z1,z2,p1,p2,T1,T2,nu,
     &     Nmol,mol_name,Nlev,alt,x,tau_h2o_cia)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to compute the optical thickness due to the H2O CIA continuum
c
c     Input:
c       + z1: lower altitude (m)
c       + z2: higher altitude (m)
c       + p1: lower pressure (atm) @ z1
c       + p2: higher pressure (atm) @ z2
c       + T1: lower temperature (K) @ z1
c       + T2: higher temperature (K) @ z2
c       + nu: wavenumber (inv. cm)
c       + Nmol:  number of species
c       + mol_name: names of molecular species
c       + Nlev: number of levels
c       + alt: altitude limit of layers (m)
c       + x: species concentrations for each level
c
c     Output:
c       + tau_h2o_cia: H2O continuum absorption optical thickness
c
c     I/O
      double precision z1,z2
      double precision p1,p2
      double precision T1,T2
      double precision nu ! inv. cm
      integer Nmol
      character*10 mol_name(1:Nmol_mx)
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision x(1:Nlev_mx,1:Nmol_mx)
      double precision tau_h2o_cia
c     temp
      integer ilev,ilay,ilev1,ilev2,band,imol
      double precision x1,x2
      logical x1_found,x2_found
      double precision k_cia_h2o1,k_cia_h2o2
      double precision nuc
      logical is_nan
      logical h2o_found,all_identical
      integer i_h2o
      double precision x_h2o(1:Nlev_mx)
      double precision kbar(1:Nlay_mx,1:Nb_mx)
      character*(Nchar_mx) kbar_file
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine H2O_continuum_optical_thickness'

c     Intentify molecule 'H2O'
      h2o_found=.false.
      do imol=1,Nmol
         if (mol_name(imol).eq.'H2O') then
            h2o_found=.true.
            i_h2o=imol
            goto 111
         endif
      enddo                     ! imol
 111  continue
      if (.not.h2o_found) then
         call error(label)
         write(*,*) 'H2O could not be identified'
         do imol=1,Nmol
            write(*,*) 'molecule index ',imol,'=',
     &           mol_name(imol)
     &           (1:strlen(mol_name(imol)))
         enddo                  ! imol
         stop
      endif                     ! h2o_found=.false.
c     At this point, molecule 'H2O' as been identified
c     "x_h2o" holds the values of the molar fraction of water for each level
      do ilev=1,Nlev
         x_h2o(ilev)=x(ilev,i_h2o)
      enddo                     ! ilev
c     compute x1, i.e. x(H2O) @ z1
      x1_found=.false.
      do ilev=1,Nlev-1
         if ((z1.ge.alt(ilev)).and.(z1.le.alt(ilev+1))) then
            x1_found=.true.
            call linear_interpolation(alt(ilev),alt(ilev+1),
     &           x_h2o(ilev),x_h2o(ilev+1),z1,x1)
c     Debug
            write(*,*) 'alt(',ilev,')=',alt(ilev),
     &           ' alt(',ilev+1,')=',alt(ilev+1),
     &           ' z1=',z1
            write(*,*) x_h2o(ilev),x_h2o(ilev+1),x1
c     Debug
            goto 221
         endif
      enddo ! ilev
      if (.not.x1_found) then
         if (z1.lt.alt(1)) then
            x1_found=.true.
            x1=x_h2o(1)
         else if (z1.gt.alt(Nlev)) then
            x1_found=.true.
            x1=x_h2o(Nlev)
         endif
      endif
 221  continue
      if (.not.x1_found) then
         call error(label)
         write(*,*) 'x1 could not be computed'
         write(*,*) 'for z1=',z1,' m'
         stop
      endif
c     compute x2, i.e. x(H2O) @ z2
      x2_found=.false.
      do ilev=1,Nlev-1
         if ((z2.ge.alt(ilev)).and.(z2.le.alt(ilev+1))) then
            x2_found=.true.
            call linear_interpolation(alt(ilev),alt(ilev+1),
     &           x_h2o(ilev),x_h2o(ilev+1),z2,x2)
c     Debug
            write(*,*) 'alt(',ilev,')=',alt(ilev),
     &           ' alt(',ilev+1,')=',alt(ilev+1),
     &           ' z2=',z2
            write(*,*) x_h2o(ilev),x_h2o(ilev+1),x2
c     Debug
            goto 222
         endif
      enddo ! ilev
      if (.not.x2_found) then
         if (z2.lt.alt(1)) then
            x2_found=.true.
            x2=x_h2o(1)
         else if (z2.gt.alt(Nlev)) then
            x2_found=.true.
            x2=x_h2o(Nlev)
         endif
      endif
 222  continue
      if (.not.x2_found) then
         call error(label)
         write(*,*) 'x2 could not be computed'
         write(*,*) 'for z2=',z2,' m'
         stop
      endif
c     Now computing CIA optical thickness
      call H2O_cia_continuum(p1*1.013D+5,T1,
     &     x1,nu,k_cia_h2o1)    ! [k_cia_h2o1]=m^-1
c     Debug
      write(*,*) 'p1=',p1*1.013D+5,
     &     ' T1=',T1,
     &     ' x1=',x1,
     &     ' nu=',nu,
     &     ' k_cia_h2o1=',k_cia_h2o1
c     Debug
      call H2O_cia_continuum(p2*1.013D+5,T2,
     &     x2,nu,k_cia_h2o2) ! [k_cia_h2o2]=m^-1
c     Debug
      write(*,*) 'p2=',p2*1.013D+5,
     &     ' T2=',T2,
     &     ' x2=',x2,
     &     ' nu=',nu,
     &     ' k_cia_h2o2=',k_cia_h2o2
c     Debug
      call tau_from_zk(z1
     &     ,z2
     &     ,k_cia_h2o1
     &     ,k_cia_h2o2
     &     ,tau_h2o_cia)
c     Debug
      write(*,*) 'z1=',z1,
     &     ' z2=',z1,
     &     ' k_cia_h2o1=',k_cia_h2o1,
     &     ' k_cia_h2o2=',k_cia_h2o2,
     &     ' tau_h2o_cia=',tau_h2o_cia
      stop
c     Debug



      
c     Debug
      call test_nan_dble(
     &     tau_h2o_cia,is_nan)
      if (is_nan) then
         call error(label)
         write(*,*) 'tau_h2o_cia=',
     &        tau_h2o_cia
         write(*,*) 'z1=',z1
         write(*,*) 'z2=',z2
         write(*,*) 'p1=',p1,' atm'
         write(*,*) 'T1=',T1,' K'
         write(*,*) 'k_cia_h2o1=',k_cia_h2o1,' m^-1'
         write(*,*) 'p2=',p2,' atm'
         write(*,*) 'T2=',T2,' K'
         write(*,*) 'k_cia_h2o2=',k_cia_h2o2,' m^-1'
         stop
      endif
c     Debug

      return
      end
