c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine solve_Tb_wavnb(nu,intensity,solution_found,Tb)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to compute the brightness temperature that fits the specified specific intensity
c     at a given wavenumber
c
c     Input:
c       + nu: wavenumber (cm^-1)
c       + intensity: specific intensity within the spectral interval (W/m^2/cm^-1/sr)
c     
c     Output:
c       + solution_found: true if a solution was found
c       + Tb: brightness temperature (K) so that B(nu,Tb)=intensity in W/m^2/cm^-1/sr
c
c     I/O
      double precision nu
      double precision intensity
      logical solution_found
      double precision Tb
c     tempcia_
      integer iter
      double precision T0,T1,Tmid
      double precision int0,int1,f0,f1,fmid
      double precision err_x,err_f,dx,df
      parameter(err_x=5.0D-5)   ! K
      parameter(err_f=1.0D-14) ! W/m^2/cm^-1/sr
      logical keep_going
      double precision dT0,dT
      double precision int1p
c     funtions
      double precision Bnu
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine solve_Tb_wavnb'

c     Debug
c      write(*,*) 'intensity=',intensity
c     Debug

      if (intensity.le.0.0D+0) then
         intensity=0.0D+0
         T0=0.0D+0
         goto 112
      endif
      
c     identify 'T0' a radiance temperature that minimizes 'intensity'
      T0=300.0D+0 ! K
      dT0=1.0D+0 ! K
      dT=dT0 ! K
      int0=Bnu(T0,nu) !  W/m^2/cm^-1/sr
      do while (int0.gt.intensity)
         T0=T0-dT ! K
         if (T0.le.0.0D+0) then
            goto 112
         endif
         int0=Bnu(T0,nu) !  W/m^2/cm^-1/sr
c     Debug
c         write(*,*) 'T0=',T0,' int0=',int0
c     Debug
         if (int0.le.0.0D+0) then
            call error(label)
            write(*,*) 'Could not compute T0:'
            write(*,*) 'intensity=',intensity
            write(*,*) 'nu=',nu
            write(*,*) 'T0=',T0,' int0=',int0
            stop
         endif
      enddo
c     Debug
c      write(*,*) 'T0=',T0,' int0=',int0
c     Debug
 112  continue

c     identify 'T1' a radiance temperature that maximizes 'intensity'
      T1=300.0D+0 ! K
      dT=dT0 ! K
      int1=Bnu(T1,nu) !  W/m^2/cm^-1/sr
      int1p=int1 !  W/m^2/cm^-1/sr
      do while (int1.lt.intensity)
         T1=T1+dT ! K
         int1=Bnu(T1,nu) !  W/m^2/cm^-1/sr
c     Debug
c         write(*,*) 'T1=',T1,' int1=',int1
c     Debug
         if (T1.gt.1000.0D+0) then
            call warning(label)
            write(*,*) 'T1=',T1
            write(*,*) 'int1=',int1
            write(*,*) 'intensity=',intensity
            solution_found=.false.
            goto 666
         endif
         if (int1.lt.int1p) then
            T1=T1-dT ! K
            dT=dT/2.0D+0 ! K
         else
            int1p=int1 !  W/m^2/cm^-1/sr
         endif
         if (dT.lt.1.0D-5) then
            call warning(label)
            write(*,*) 'Impossible to find T1 for intensity=',intensity
            write(*,*) 'Maximum reached:'
            write(*,*) 'T1=',T1,' int1=',int1
            write(*,*) 'now using intensity=',int1
            intensity=int1
            goto 111
         endif
      enddo
 111  continue
c     Debug
c      write(*,*) 'T1=',T1,' int1=',int1
c     Debug

c     Dichotomy procedure to find a precise value
      keep_going=.true.
      solution_found=.false.
      iter=0
      do while (keep_going)
         iter=iter+1
         f0=Bnu(T0,nu) !  W/m^2/cm^-1/sr
         f1=Bnu(T1,nu) !  W/m^2/cm^-1/sr
         Tmid=(T0+T1)/2.0D+0 ! K
         fmid=Bnu(Tmid,nu) !  W/m^2/cm^-1/sr
         if (fmid.lt.intensity) then
            T0=Tmid ! K
         else
            T1=Tmid ! K
         endif
         dx=dabs(T1-T0)
         df=dabs(f1-f0)
         if ((dx.lt.err_x).or.(df.lt.err_f)) then
            keep_going=.false.
            solution_found=.true.
            Tb=Tmid ! K
c     Debug
c            write(*,*) 'solution found !'
c            write(*,*) 'Tb=',Tb
c            write(*,*) 'intensity=',intensity
c            write(*,*) 'fmid=',fmid
c     Debug
         endif
         if (iter.gt.Niter_mx) then
            keep_going=.false.
            solution_found=.false.
         endif
      enddo ! while ! keep_going

 666  continue
      if (.not.solution_found) then
         call error(label)
         write(*,*) 'Solution could not be found:'
         write(*,*) 'nu=',nu
         write(*,*) 'intensity=',intensity
         write(*,*) 'iter=',iter
         write(*,*) 'dx=',dx
         write(*,*) 'df=',df
         write(*,*) 'T0=',T0,' f0=',f0
         write(*,*) 'T1=',T1,' f1=',f1
c         stop
      endif

      return
      end



      subroutine net_flux_toa_to_obs_intensity(ai,net_flux_toa,
     &     obs_intensity)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to compute the intensity (W/m^2/sr) perceived by a remote detector
c     from the TOA net flux (W/m^2 at top of the atmosphere). This net flux @ TOA is
c     assumed to be homogeneously dispersed over 2pi steradians.
c
c     Input:
c       + ai: true if angular integration is enabled
c       + net_flux_toa: net flux at the top of the atmosphere (W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F)
c     
c     Output:
c       + obs_intensity: intensity perceived by a remote detector (W/m^2/cm^-1/sr)
c         I mean Watts perceived by square meter of detector surface,
c         per steradian of solid angle covered by the detector.
c
c     I/O
      logical ai
      double precision net_flux_toa
      double precision obs_intensity
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine net_flux_toa_to_obs_intensity'

      if (net_flux_toa.lt.0.0D+0) then
         call warning(label)
         write(*,*) 'net_flux_toa=',net_flux_toa
         write(*,*) 'is negative'
      endif
      if (ai) then
         obs_intensity=net_flux_toa/pi
      else
         obs_intensity=net_flux_toa
      endif
      
      return
      end

