c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine cia_interpolation(ndt,nu_min,nu_max,np,T,cia,
     &     nu,temperature,sigma)
      implicit none
      include 'max.inc'
c
c     Purpose: to interpolate CIA data from a HITRAN2012 table
c     at a given wavenumber / temperature
c     
c     Input:
c       + ndt: number of spectral intervals / temperature points the data is provided for
c       + nu_min: lower wavenumber (inv. cm) for each data interval
c       + nu_max: higher wavenumber (inv. cm) for each data interval
c       + np: number of wavenumber points for each data interval
c       + T: temperature (K) for each data interval
c       + cia: CIA data for each data interval, for each wavenumber point
c         - column 1: wavenumber (inv. cm)
c         - column 2: CIA cross-section (cm^5.molecule^-2)
c       + nu: interpolation wavenuber (inv. cm)
c       + temperature: interpolation temperature
c
c     Output:
c       + sigma: CIA cross-section (cm^5.molecule^-2) @ (nu,temperature)
c
      double precision nu_min(1:Ndt_int_mx)
      double precision nu_max(1:Ndt_int_mx)
      integer ndt,np(1:Ndt_int_mx)
      double precision T(1:Ndt_int_mx)
      double precision cia(1:Ndt_int_mx,1:Np_mx,1:2)
      double precision nu
      double precision temperature
      double precision sigma
c     temp
      integer N_nu_int
      logical nu_int_found
      integer nu_int_number(1:Ndt_int_mx)
      integer nu_int_index(1:Ndt_int_mx,1:Ndt_int_mx)
      double precision nu_int_numin(1:Ndt_int_mx)
      double precision nu_int_numax(1:Ndt_int_mx)
      double precision nu_int_Tmin(1:Ndt_int_mx)
      double precision nu_int_Tmax(1:Ndt_int_mx)
      integer Nvalid_nu
      logical valid_nu(1:Ndt_int_mx)
      integer Nvalid_t
      logical valid_t(1:Ndt_int_mx)
      double precision dT1,dT2,deltaT
      integer nu_int
      integer idt,i,j
      logical it_found
      integer it,it1,it2
      logical inu_found
      integer inu,inu1,inu2
      double precision sigma_t1,sigma_t2
c     parameters
      double precision eps_nu
      parameter(eps_nu=1.0D-6)
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine cia_interpolation'

c     Identify the number of wavenumber intervals contained in the CIA table
      N_nu_int=0
      do i=1,Ndt_int_mx
         nu_int_number(i)=0
         do j=1,Ndt_int_mx
            nu_int_index(i,j)=0
         enddo ! j
      enddo ! i

      do idt=1,ndt
         nu_int_found=.false.
         do i=1,N_nu_int
            if ((dabs(nu_min(idt)-nu_int_numin(i)).lt.eps_nu).and.
     &           (dabs(nu_max(idt)-nu_int_numax(i)).lt.eps_nu)) then
               nu_int_found=.true.
c     Data interval index "idt" has already been identified as wavenumber interval index "i"
               nu_int_number(i)=nu_int_number(i)+1
c     "nu_int_number(i)" is the number of data intervals for wavenumber interval index "i"
               if (nu_int_number(i).gt.Ndt_int_mx) then
                  call error(label)
                  write(*,*) 'nu_int_number(',i,')=',nu_int_number(i)
                  write(*,*) 'while Ndt_int_mx=',Ndt_int_mx
                  stop
               endif
c     "nu_int_index(i,nu_int_number(i))" is the index of the data interval
               nu_int_index(i,nu_int_number(i))=idt
c     update "nu_int_Tmin(i)" and "nu_int_Tmax(i)" if necessary
               if (T(idt).lt.nu_int_Tmin(i)) then
                  nu_int_Tmin(i)=T(idt)
               endif
               if (T(idt).gt.nu_int_Tmax(i)) then
                  nu_int_Tmax(i)=T(idt)
               endif
c
               goto 111
            endif
         enddo ! i
 111     continue
         if (.not.nu_int_found) then
c     If data interval index "idt" has not been identified as an existing wavenumber interval,
c     one should be created with all relevant data
            N_nu_int=N_nu_int+1
            if (N_nu_int.gt.Ndt_int_mx) then
               call error(label)
               write(*,*) 'N_nu_int=',N_nu_int
               write(*,*) 'while Ndt_int_mx=',Ndt_int_mx
               stop
            endif
            nu_int_number(N_nu_int)=nu_int_number(N_nu_int)+1
            nu_int_index(N_nu_int,nu_int_number(N_nu_int))=idt
            nu_int_numin(N_nu_int)=nu_min(idt)
            nu_int_numax(N_nu_int)=nu_max(idt)
            nu_int_Tmin(N_nu_int)=T(idt)
            nu_int_Tmax(N_nu_int)=T(idt)
         endif ! (.not.nu_int_found)
c
      enddo ! idt

c     Debug
c      write(*,*) 'N_nu_int=',N_nu_int
c      do i=1,N_nu_int
c         write(*,*) 'wavenumber interval index:',i
c         write(*,*) 'nu_min=',nu_int_numin(i),' nu_max=',nu_int_numax(i)
c         write(*,*) 'Number if data intervals',nu_int_number(i)
c         write(*,*) 'data intervals:',(nu_int_index(i,j)
c     &        ,j=1,nu_int_number(i))
c         write(*,*) 'min. temperature:',nu_int_Tmin(i)
c         write(*,*) 'max. temperature:',nu_int_Tmax(i)
c         write(*,*)
c      enddo ! i
c     Debug

c     Look for wavenumber intervals that match "nu"
      Nvalid_nu=0
      do i=1,N_nu_int
         valid_nu(i)=.false.
         if ((nu_int_numin(i).le.nu).and.(nu_int_numax(i).ge.nu)) then
            Nvalid_nu=Nvalid_nu+1
            valid_nu(i)=.true.
         endif
      enddo ! i
      nu_int_found=.false.
      if (Nvalid_nu.eq.0) then
c     If no wavenumber interval contain CIA data for wavenumber "nu", then return sigma=0
         sigma=0.0D+0
         goto 666
      else if (Nvalid_nu.eq.1) then
c     If there is exactly one wavenumber interval containing CIA data for "nu", then use this one
         do i=1,N_nu_int
            nu_int_found=.true.
            if (valid_nu(i)) then
               nu_int=i
            endif
         enddo ! i
      else
c     If there are multiple wavenumber intervals that match "nu", then things are a
c     bit more complicated:
c     - first, see if there is a temperature range that match "temperature"
         Nvalid_t=0
         do i=1,N_nu_int
            valid_t(i)=.false.
            if ((valid_nu(i)).and.
     &           ((nu_int_Tmin(i).le.temperature).and.
     &           (nu_int_Tmax(i).ge.temperature))) then
               Nvalid_t=Nvalid_t+1
               valid_t(i)=.true.
            endif
         enddo ! i
         if (Nvalid_t.eq.0) then
c     - if there is no wavenumber interval with a matching temperature range, then
c     we will use the wavenumber interval with the temperature range closest to "temperature"
            deltaT=10000.0D+0
            do i=1,N_nu_int
               if (valid_nu(i)) then
                  dT1=dabs(temperature-nu_int_Tmin(i))
                  dT2=dabs(temperature-nu_int_Tmax(i))
                  if (dT1.lt.deltaT) then
                     deltaT=dT1
                     nu_int_found=.true.
                     nu_int=i
                  endif
                  if (dT2.lt.deltaT) then
                     deltaT=dT2
                     nu_int_found=.true.
                     nu_int=i
                  endif
               endif ! valid_nu(i)
            enddo ! i
            if (.not.nu_int_found) then
               call error(label)
               write(*,*) 'Several wavenumber intervals possible:'
               write(*,*) 'nu=',nu
               do i=1,N_nu_int
                  if (valid_nu(i)) then
                     write(*,*) 'wavenumber interval index:',i
                     write(*,*) 'nu range=[',nu_int_numin(i),
     &                    ',',nu_int_numax(i),']'
                     write(*,*) 'T range=[',nu_int_Tmin(i),
     &                    ',',nu_int_Tmax(i),']'
                  endif
               enddo ! i
               write(*,*) 'But not suitable interval found for ',
     &              'temperature=',temperature
               stop
            endif ! .not.nu_int_found
         else if (Nvalid_t.eq.1) then
c     - if there is exactly one wavenumber interval with a matching temperature range, then
c     use this one !
            do i=1,N_nu_int
               if ((valid_nu(i)).and.(valid_t(i))) then
c     Debug
c                  write(*,*) 'valid_nu(',i,')=',valid_nu(i),
c     &                 ' valid_t(',i,')=',valid_t(i)
c     Debug
                  nu_int_found=.true.
                  nu_int=i
               endif
            enddo ! i
         else
c     - if there is more than one wavenumber interval with a matching temperature range, then
c     display a error message (this should not happen)
            call error(label)
            write(*,*) 'Several wavenumber intervals possible:'
            write(*,*) 'nu=',nu
            write(*,*) 'temperature=',temperature
            do i=1,N_nu_int
               if ((valid_nu(i)).and.(valid_t(i))) then
                  write(*,*) 'wavenumber interval index:',i
                  write(*,*) 'nu range=[',nu_int_numin(i),
     &                 ',',nu_int_numax(i),']'
                  write(*,*) 'T range=[',nu_int_Tmin(i),
     &                 ',',nu_int_Tmax(i),']'
               endif
            enddo               ! i
            write(*,*) 'There is more than 1 possible interval'
            stop            
         endif                  ! Nvalid_t
      endif
c     Debug
c      write(*,*) 'nu_int_found=',nu_int_found
c      if (nu_int_found) then
c         write(*,*) 'nu_int=',nu_int
c         write(*,*) 'nu range=[',nu_int_numin(nu_int),
c     &        ',',nu_int_numax(nu_int),']'
c         write(*,*) 'T range=[',nu_int_Tmin(nu_int),
c     &        ',',nu_int_Tmax(nu_int),']'
c      endif
c     Debug

      if (.not.nu_int_found) then
         call error(label)
         write(*,*) 'nu_int_found=',nu_int_found
         stop
      endif
c     Once the wavenumber interval to use has been found:
c     - the temperature levels used for interpolation must be identified
      it_found=.false.
      if ((nu_int_Tmin(nu_int).le.temperature).and.
     &     (nu_int_Tmax(nu_int).ge.temperature)) then
         do i=1,nu_int_number(nu_int)-1
            if ((T(nu_int_index(nu_int,i)).le.temperature).and.
     &           (T(nu_int_index(nu_int,i+1)).ge.temperature)) then
               it_found=.true.
               it1=i
               it2=i+1
            endif
         enddo ! i
      else if (nu_int_Tmin(nu_int).ge.temperature) then
         it_found=.true.
         it1=1
         it2=2
      else if (nu_int_Tmax(nu_int).le.temperature) then
         it_found=.true.
         it1=nu_int_number(nu_int)-1
         it2=nu_int_number(nu_int)
      endif
      if (.not.it_found) then
         call error(label)
         write(*,*) 'Temperature levels could not be found'
         write(*,*) 'for wavenumber index:',nu_int
         write(*,*) 'T range=[',nu_int_Tmin(nu_int),
     &        ',',nu_int_Tmax(nu_int),']'
         write(*,*) 'temperature=',temperature
         stop
      endif

c     Debug
c      write(*,*) 'temperature=',temperature
c      write(*,*) 'T(',nu_int,',',it1,')=',T(nu_int_index(nu_int,it1))
c      write(*,*) 'T(',nu_int,',',it2,')=',T(nu_int_index(nu_int,it2))
c     Debug

c     Perform a linear interpolation over wavenumber for temperature level 1
      inu_found=.false.
      do inu=1,np(nu_int_index(nu_int,it1))-1
         if ((cia(nu_int_index(nu_int,it1),inu,1).le.nu).and.
     &        (cia(nu_int_index(nu_int,it1),inu+1,1).ge.nu)) then
            inu_found=.true.
            sigma_t1=cia(nu_int_index(nu_int,it1),inu,2)
     &           +(cia(nu_int_index(nu_int,it1),inu+1,2)
     &           -cia(nu_int_index(nu_int,it1),inu,2))
     &           /(cia(nu_int_index(nu_int,it1),inu+1,1)
     &           -cia(nu_int_index(nu_int,it1),inu,1))
     &           *(nu
     &           -cia(nu_int_index(nu_int,it1),inu,1))
         endif
      enddo ! inu
      if (.not.inu_found) then
         call error(label)
         write(*,*) 'sigma could not be interpolated for:'
         write(*,*) 'T(',nu_int,',',it1,')=',
     &        T(nu_int_index(nu_int,it1))
         write(*,*) 'nu=',nu
         write(*,*) 'nu range=[',nu_int_numin(nu_int),
     &        ',',nu_int_numax(nu_int),']'
         stop
      endif
c     Perform a linear interpolation over wavenumber for temperature level 2
      inu_found=.false.
      do inu=1,np(nu_int_index(nu_int,it2))-1
         if ((cia(nu_int_index(nu_int,it2),inu,1).le.nu).and.
     &        (cia(nu_int_index(nu_int,it2),inu+1,1).ge.nu)) then
            inu_found=.true.
            sigma_t2=cia(nu_int_index(nu_int,it2),inu,2)
     &           +(cia(nu_int_index(nu_int,it2),inu+1,2)
     &           -cia(nu_int_index(nu_int,it2),inu,2))
     &           /(cia(nu_int_index(nu_int,it2),inu+1,1)
     &           -cia(nu_int_index(nu_int,it2),inu,1))
     &           *(nu
     &           -cia(nu_int_index(nu_int,it2),inu,1))
         endif
      enddo ! inu
      if (.not.inu_found) then
         call error(label)
         write(*,*) 'sigma could not be interpolated for:'
         write(*,*) 'T(',nu_int,',',it2,')=',
     &        T(nu_int_index(nu_int,it2))
         write(*,*) 'nu=',nu
         write(*,*) 'nu range=[',nu_int_numin(nu_int),
     &        ',',nu_int_numax(nu_int),']'
         stop
      endif

c     Debug
c      write(*,*) 'sigma_t1=',sigma_t1
c      write(*,*) 'sigma_t2=',sigma_t2
c     Debug

c     Perform a linear interpolation over temperature
      sigma=sigma_t1
     &     +(sigma_t2-sigma_t1)
     &     /(T(nu_int_index(nu_int,it2))-T(nu_int_index(nu_int,it1)))
     &     *(temperature-T(nu_int_index(nu_int,it1)))
      
 666  continue
c     Debug
c      write(*,*) 'sigma=',sigma
c     Debug
      return
      end
