c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine profiles(debug,use_cloud,add_sca2abs,
     &     Nlev,nu,k,alt,pres,temp,Tg,Ts,abs_cont,ks_coeff,
     &     tau_rayleigh_1k,tau_co2_cia_1k,tau_h2o_cia_1k,
     &     tau_N2N2_cia_1k,tau_O2O2_cia_1k,tau_h2o_cont_1k,
     &     Nb_cloud,nu_lo_cloud,nu_hi_cloud,ka_cloud,ks_cloud,g_cloud,
     &     Nlay,Bg,Bs,B,tau)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to compute Planck intensity for the ground, for space, at each
c     interface, and the optical thickness of each gas layer, for a given
c     wavenumber and for a given profile of k at that wavenumber
c
c     Input:
c       + use_cloud: if true, cloud data will be used
c       + add_sca2abs: if true, scattering is added to the absorption optical thicnkess
c       + Nlev: number of levels (interfaces) k is given for
c       + nu: wavenumber (inverse centimeters)
c       + k: vertical profile of ka (inverse meters)
c       + alt: altitude at each level (meters)
c       + pres: pressure profile as read by "read_composition" (atm)
c       + temp: temperature profile as read by "read_composition" (K)
c       + Tg: ground temperature
c       + Ts: space temperature
c       + abs_cont: absorption continuum (cm^-1/amg^2)
c       + ks_coeff: division factor for scattering optical thicknesses
c       + tau_rayleigh_1k: rayleigh scattering optical thickness
c       + tau_co2_cia_1k: collision-induced absorption for CO2
c       + tau_h2o_cia_1k: collision-induced absorption for H2O
c       + tau_N2N2_cia_1k: N2-N2 collision-induced absorption
c       + tau_O2O2_cia_1k: O2-O2 collision-induced absorption
c       + tau_h2o_cont_1k: H2O continuum absorption optical thickness
c       + Nb_cloud: number of cloud spectral intervals
c       + nu_lo_cloud: lower wavenumber of each cloud interval (inv. cm)
c       + nu_hi_cloud: lower wavenumber of each cloud interval (inv. cm)
c       + ka_cloud: cloud absorbtion coefficient (inv. m) for each interval, for each level
c       + ks_cloud: cloud scattering coefficient (inv. m) for each interval, for each level
c       + g_cloud: cloud asymetry parameter for each interval, for each level
c
c     Output:
c       + Nlay: number of gas layers
c       + Bg: Planck intensity of the ground (W/m^2/sr/cm^-1)
c       + Bs: Planck intensity of space (W/m^2/sr/cm^-1)
c       + B: vertical profile of Planck intensity (W/m^2/sr/cm^-1) at each level
c       + tau: optical thickness of each gas layer
c
c     I/O
      logical debug
      logical use_cloud,add_sca2abs
      integer Nlev
      double precision nu
      double precision k(1:Nlev_mx)
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision temp(1:Nlev_mx)
      double precision Tg,Ts
      double precision abs_cont
      double precision ks_coeff
      double precision tau_rayleigh_1k(1:Nlay_mx)
      double precision tau_co2_cia_1k(1:Nlay_mx)
      double precision tau_h2o_cia_1k(1:Nlay_mx)
      double precision tau_N2N2_cia_1k(1:Nlay_mx)
      double precision tau_O2O2_cia_1k(1:Nlay_mx)
      double precision tau_h2o_cont_1k(1:Nlay_mx)
      integer Nb_cloud
      double precision nu_lo_cloud(1:Nb_mx)
      double precision nu_hi_cloud(1:Nb_mx)
      double precision ka_cloud(1:Nlev_mx,1:Nb_mx)
      double precision ks_cloud(1:Nlev_mx,1:Nb_mx)
      double precision g_cloud(1:Nlev_mx,1:Nb_mx)
      integer Nlay
      double precision Bg,Bs,B(0:Nlay_mx)
      double precision tau(1:Nlay_mx)
c     spline integration
      integer Np
      double precision x(1:Np_mx)
      double precision f(1:Np_mx)
      integer lev1,lev2
c     temp
      double precision T(0:Nlay_mx)
      double precision Bnu
      double precision kac(1:Nlay_mx)
      double precision ksc(1:Nlay_mx)
      double precision gc(1:Nlay_mx)
      double precision z1,z2,k1,k2
      double precision tau_gas(1:Nlay_mx)
      double precision tau_abs_cloud(1:Nlay_mx)
      double precision tau_sca_cloud(1:Nlay_mx)
      double precision amg
      double precision ka_cont1,ka_cont2
      double precision taug,tauc,tau_atm
      integer i,j,lay,band
      logical sint_found
      integer sint
      logical is_nan
      character*(Nchar_mx) input_file,res_file
      logical mpf_ex
c     parameters
      double precision tau_min
      parameter(tau_min=1.0D-5)
c     Debug
      integer emitting_layer
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine profiles'

c     security
      do i=2,Nlev
         if (alt(i).le.alt(i-1)) then
            call error(label)
            write(*,*) 'altitude profile is not suitable'
            write(*,*) 'for optical depths computation'
            stop
         endif
      enddo ! i
c     security

      Nlay=Nlev-1
      call Tprofile(Nlay,temp,T)
      call Bprofile(Nlay,Tg,T,Ts,nu,Bg,B,Bs)
c
c     Debug
c      if (debug) then
c         open(10,file='./results/TB_profile.txt')
c         write(10,*) 'nu=',nu
c         write(10,*) 'Tground=',Tg,' Bg=',Bg
c         write(10,*) 'Tspace=',Ts,' Bs=',Bs
c         write(10,*) 'Gas: i / T(K) / B(W/m^2/sr/cm^-1)'
c         do i=0,Nlay
c            write(10,*) i,T(i),B(i)
c         enddo ! i
c         close(10)
c      endif
c     Debug

      tau_atm=0.0D+0
      do i=1,Nlay
c     gas opacity
         if ((k(i).eq.0.0D+0).and.(k(i+1).eq.0.0D+0)) then
            taug=0.0D+0
         else if ((k(i).gt.0.0D+0).and.(k(i+1).eq.0.0D+0)) then
            taug=k(i)*(alt(i+1)-alt(i))/2.0D+0
         else if ((k(i).gt.0.0D+0).and.(k(i+1).gt.0.0D+0)) then
            taug=(alt(i+1)-alt(i))
     &           *(k(i)-k(i+1))
     &           /dlog(k(i)/k(i+1))
         endif
c     additional continuum opacity
         call amagat(pres(i),temp(i),amg)
         ka_cont1=abs_cont*1.0D+2*amg**2.0D+0 ! inv. meters
         call amagat(pres(i+1),temp(i+1),amg)
         ka_cont2=abs_cont*1.0D+2*amg**2.0D+0 ! inv. meters
c     tau_gas(i)=int(alt(i),alt(i+1),ka(z)dz)
c     with an hypothesis of exponential evolution of k(z):
c     k(alt(i))=k(i)
c     k(alt(i+1))=k(i+1)
c     k(z)=k(i)*exp(-alpha*(z-alt(i))/(alt(i+1)-alt(i))) with alpha=ln(k(i)/k(i+1))
c     => tau=int(k(z)dz,z=alt(i),z=alt(i+1))=(alt(i+1)-alt(i))*(k(i)-k(i+1))/ln(k(i)/k(i+1))
c         k1=k(i)+ka_cont1
c         k2=k(i+1)+ka_cont2
         if (abs_cont.le.0.0D+0) then
            tauc=0.0D+0
         else
            tauc=(alt(i+1)-alt(i))
     &           *(ka_cont1-ka_cont2)
     &           /dlog(ka_cont1/ka_cont2)
         endif
c     Debug
c         write(*,*) 'tauc(',i,')=',tauc
c     Debug
c     total opacity
         tau_gas(i)=taug ! gas optical thickness
     &        +tauc ! continuum optical thickness
     &        +tau_co2_cia_1k(i) ! collision-induced absorption for CO2
     &        +tau_h2o_cia_1k(i) ! collision-induced absorption for H2O
     &        +tau_N2N2_cia_1k(i) ! N2-N2 collision-induced absorption
     &        +tau_O2O2_cia_1k(i) ! O2-O2 collision-induced absorption
     &        +tau_h2o_cont_1k(i) ! H2O continuum absorption
         if (add_sca2abs) then ! only if required
            tau_gas(i)=tau_gas(i)
     &           +tau_rayleigh_1k(i) ! Rayleigh scattering optical thickness
     &           /ks_coeff
         endif
         tau_atm=tau_atm+tau_gas(i)
      enddo                     ! i
c     Debug
c      res_file='./results/gs_tau.txt'
c      open(19,file=res_file(1:strlen(res_file)),access='append')
c      write(19,*) nu,tau_atm
c      close(19)
c     Debug

      if (use_cloud) then
c     Cloud data
c     1 - locating the cloud spectral interval
         sint_found=.false.
         do band=1,Nb_cloud
            if ((nu_lo_cloud(band).le.nu).and.
     &           (nu_hi_cloud(band).ge.nu)) then
               sint_found=.true.
               sint=band
               goto 111
            endif
         enddo                  ! i
 111     continue
         if (.not.sint_found) then
            call error(label)
            write(*,*) 'Cloud spectral interval could not be found'
            write(*,*) 'for nu=',nu
            write(*,*) 'nu_lo_cloud(1)=',nu_lo_cloud(1)
            write(*,*) 'nu_hi_cloud(',Nb_cloud,')=',
     &           nu_hi_cloud(Nb_cloud)
            stop
         endif
c     2 - get data for vertical integration
         do i=1,Nlev
            kac(i)=ka_cloud(i,sint)
            ksc(i)=ks_cloud(i,sint)
            gc(i)=g_cloud(i,sint)
         enddo                  ! i

         do lay=1,Nlay
            lev1=lay
            lev2=lay+1
            z1=alt(lev1)
            z2=alt(lev2)
c     absorption coefficient
            k1=kac(lev1)
            k2=kac(lev2)
            call tau_from_zk(z1,z2,k1,k2,tau_abs_cloud(lay))
c     Debug
            call test_nan_dble(tau_abs_cloud(lay),is_nan)
            if (is_nan) then
               call error(label)
               write(*,*) 'tau_abs_cloud(',lay,')=',
     &              tau_abs_cloud(lay)
               write(*,*) 'z1=',z1
               write(*,*) 'z2=',z2
               write(*,*) 'k1=',k1
               write(*,*) 'k2=',k2
               stop
            endif
c     Debug
c     scattering coefficient
            k1=ksc(lev1)
            k2=ksc(lev2)
            call tau_from_zk(z1,z2,k1,k2,tau_sca_cloud(lay))
c     Debug
            call test_nan_dble(tau_sca_cloud(lay),is_nan)
            if (is_nan) then
               call error(label)
               write(*,*) 'tau_sca_cloud(',lay,')=',
     &              tau_sca_cloud(lay)
               stop
            endif
c     Debug
         enddo                  ! lay
      else                      ! use_cloud=false
         do lay=1,Nlay
            tau_abs_cloud(lay)=0.0D+0
            tau_sca_cloud(lay)=0.0D+0
         enddo ! lay
      endif

c     Add cloud optical thickness to gas optical thickness
      do i=1,Nlay
         tau(i)=tau_gas(i)+tau_abs_cloud(i)
         if (add_sca2abs) then ! only if required
            tau(i)=tau(i)
     &           +tau_sca_cloud(i)*(1.0D+0-gc(i))
     &           /ks_coeff
         endif
      enddo                     ! i
c     Debug
c      emitting_layer=48
c      do i=1,Nlay
c         if (i.eq.emitting_layer) then
c            tau(i)=1.0D+6
c         else
c            tau(i)=0.0D+0
c         endif
c      enddo                     ! i
c      write(*,*) 'T(',emitting_layer,')=',T(emitting_layer)
c      write(*,*) 'alt=',alt(emitting_layer),
c     &     '-',alt(emitting_layer+1)
c      write(*,*) 'z=',(alt(emitting_layer)
c     &     +alt(emitting_layer+1))/2.0D+0
c     Debug

c     Debug
c      if (debug) then
c         open(10,file='./results/tau_profile.txt')
c         write(10,10) 'Tau profile for nu='
c         write(10,*) nu
c         write(10,10) 'z_inf / z_sup / tau_gas / tau_cloud / tau'
c         do i=1,Nlay
c            write(10,*) alt(i),alt(i+1),tau_gas(i)
c     &           ,tau_abs_cloud(i),tau_sca_cloud(i),tau(i)
c         enddo ! i
c         close(10)
c      endif
c     Debug

      do i=1,Nlay
c     security for not deviding by zero...
c     !!!! WARNING !!!!!
c     since a null value will crash the code (devide by tau), a very small value
c     must be given to tau(i); 1e-12 is the smallest possible value that should be used:
c     in practice, values smaller than "tau_min" will cause a bad behaviour of the expint() function.
         if (tau(i).lt.tau_min) then
            tau(i)=tau_min
         endif
      enddo ! i

      return
      end



      subroutine Tprofile(Nlay,temp,T)
      implicit none
      include 'max.inc'
c     
c     Purpose: obtain the temperature at each interface
c     
c     Input:
c       + Nlay: number of atmospheric layers
c       + temp: interface temperatures
c     
c     Output:
c       + T: temperature of each interface [K]
c     
c     I/O
      integer Nlay
      double precision temp(1:Nlev_mx)
      double precision T(0:Nlay_mx)
c     temp
      integer i
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine Tprofile'
      
      do i=0,Nlay
         T(i)=temp(i+1) ! K
      enddo

      return
      end



      subroutine mean_Tprofile(Nlay,temp,T)
      implicit none
      include 'max.inc'
c     
c     Purpose: obtain the mean temperature in each gas layer
c     
c     Input:
c       + Nlay: number of atmospheric layers
c       + temp: interface temperatures
c     
c     Output:
c       + T: mean temperature of each gas layer [K]
c     
c     I/O
      integer Nlay
      double precision temp(1:Nlev_mx)
      double precision T(0:Nlay_mx)
c     temp
      integer i
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine mean_Tprofile'
      
      T(0)=temp(1)              ! K
      do i=1,Nlay
         T(i)=(temp(i)+temp(i+1))/2.0D+0 ! K
      enddo                     ! i

      return
      end


      

      subroutine Bprofile(Nlay,Tg,T,Ts,nu,Bg,B,Bs)
      implicit none
      include 'max.inc'
c     
c     Purpose: obtain the mean Blackbody intensity at each interface
c     
c     Input:
c       + Nlay: number of atmospheric layers
c       + Tg: ground temperature [K]
c       + T: mean temperature at each interface [K]
c       + Ts: space temperature [K]
c       + nu: wavenumber [cm^-1]
c     
c     Output:
c       + Bg: ground blackbody intensity [W/m^2/sr/cm^-1]
c       + B: blackbody intensity at each interface [W/m^2/sr/cm^-1]
c       + Bs: space blackbody intensity [W/m^2/sr/cm^-1]
c     
c     I/O
      integer Nlay
      double precision Tg
      double precision T(0:Nlay_mx)
      double precision Ts
      double precision nu
      double precision Bg
      double precision B(0:Nlay_mx)
      double precision Bs
c     temp
      integer i
c     functions
      double precision Bnu
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine Bprofile'
      
      Bg=Bnu(Tg,nu) ! W/m^2/sr/cm^-1
      Bs=Bnu(Ts,nu) ! W/m^2/sr/cm^-1
      do i=0,Nlay
         B(i)=Bnu(T(i),nu) ! W/m^2/sr/cm^-1
      enddo

      return
      end


      

      subroutine mean_Bprofile(Nlay,Tg,T,Ts,nu,Bg,B,Bs)
      implicit none
      include 'max.inc'
c     
c     Purpose: obtain the mean Blackbody intensity in each gas layer
c     
c     Input:
c       + Nlay: number of atmospheric layers
c       + Tg: ground temperature [K]
c       + T: mean temperature of each gas layer [K]
c       + Ts: space temperature [K]
c       + nu: wavenumber [cm^-1]
c     
c     Output:
c       + Bg: ground blackbody intensity [W/m^2/sr/cm^-1]
c       + B: mean blackbody intensity of each atmospheric layer [W/m^2/sr/cm^-1]
c       + Bs: space blackbody intensity [W/m^2/sr/cm^-1]
c     
c     I/O
      integer Nlay
      double precision Tg
      double precision T(0:Nlay_mx)
      double precision Ts
      double precision nu
      double precision Bg
      double precision B(0:Nlay_mx)
      double precision Bs
c     temp
      integer i
c     functions
      double precision Bnu
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine mean_Bprofile'
      
      Bg=Bnu(Tg,nu) ! W/m^2/sr/cm^-1
      Bs=Bnu(Ts,nu) ! W/m^2/sr/cm^-1
      do i=0,Nlay
         B(i)=Bnu(T(i),nu) ! W/m^2/sr/cm^-1
      enddo

      return
      end
      


      subroutine identify_probe_altitude(Nlev,Nlay,
     &     alt,pres,probe_pressure,
     &     probe_layer,probe_altitude)
      implicit none
      include 'max.inc'
c      
c      Purpose: to identify the layer/altitude corresponding to the probe pressure
c      
c      Input:
c        + Nlay: number of layers
c        + alt: altitude array (m)
c        + pres: pressure array (atm)
c        + probe_pressure: probe pressure (atm)
c      
c      Output:
c        + probe_layer: atmospheric layer where the probe pressure is located
c        + probe_altitude: altitude corresponding to the probe pressure (m)
c      
c     I/O
      integer Nlev,Nlay
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision probe_pressure
      integer probe_layer
      double precision probe_altitude
c     temp
      integer ilay
      logical probe_layer_found
      double precision alpha,p1,p2,z1,z2
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine identify_probe_altitude'

      probe_layer_found=.false.
      do ilay=1,Nlay
         if ((probe_pressure.le.pres(ilay)).and.
     &        (probe_pressure.ge.pres(ilay+1))) then
            probe_layer_found=.true.
            probe_layer=ilay
            goto 111
         endif
      enddo                     ! ilay
 111  continue
      if (.not.probe_layer_found) then
         call error(label)
         write(*,*) 'probe layer could not be found'
         write(*,*) 'probe_pressure=',probe_pressure,' atm'
         write(*,*) 'layer index / higher pressure / lower pressure'
         do ilay=1,Nlay
            write(*,*) ilay,pres(ilay),pres(ilay+1)
         enddo                  ! ilay
         stop
      else
         z1=alt(probe_layer)
         z2=alt(probe_layer+1)
         p1=pres(probe_layer)
         p2=pres(probe_layer+1)
         alpha=dlog(p1/p2)
         probe_altitude=z1+(z2-z1)*dlog(p1/probe_pressure)/alpha
      endif

      return
      end
      
